#pragma once

#include "request.h"

namespace cppmicrohttpd
{

    request::request(
        MHD_Connection * const  p_connection,
        const std::string&      p_url,
        const std::string&      p_method,
        const std::string&      p_version)
        : connection(p_connection)
        , url       (p_url)
        , method    (p_method)
        , version   (p_version)
        { }

    std::string request::get_header(const std::string& key) const
    {
        std::string value;
        get_header(key, value);
        return value;
    }

    bool request::get_header(const std::string& key, std::string& value) const
    {
        auto v = MHD_lookup_connection_value(connection, MHD_HEADER_KIND, key.c_str());
        if (v) value.assign(v);
        return static_cast<bool>(v);
    }

    template<typename T_exception>
    void request::set_error(const T_exception& p_error)
    {
        if (!error)
            error.reset(new T_exception(p_error));
    }

}
