#pragma once

#include <memory>
#include <microhttpd.h>

namespace cppmicrohttpd
{

    struct request;
    struct response;

    using response_ptr_u = std::unique_ptr<response>;

    struct response
    {
    public:
        using handle_ptr_u = std::unique_ptr<MHD_Response, decltype(&MHD_destroy_response)>;
        using request_t    = ::cppmicrohttpd::request;

    public:
        const request_t&    request;        //!< Request this response belongs to.
        handle_ptr_u        handle;         //!< MHD response handle
        unsigned int        status { 0 };   //!< HTTP status code

    public:
        /**
         * @brief Constructor.
         *
         * @param[in] p_request     Request this response belongs to.
         */
        inline response(const request_t& p_request);

        /**
         * @brief Destructor.
         */
        virtual ~response() = default;

        /**
         * @brief Add header entry to the response.
         *
         * @param[in] key   Header key.
         * @param[in] value Header value.
         */
        inline void add_header(const char * key, const char * value);

    public:
        /**
         * @brief Create a simple HTML response.
         *
         * @param[in] p_request     Reference to request.
         * @param[in] p_status      HTTP status of the response.
         * @param[in] p_title       Title of the HTML document.
         * @param[in] p_head        Headline of the HTML document.
         * @param[in] p_msg         Content of the HTML document.
         *
         * @return              Created response.
         */
        static response_ptr_u build_simple_html_response(
            const cppmicrohttpd::request& p_request,
            unsigned int p_status,
            const std::string& p_title,
            const std::string& p_head,
            const std::string& p_msg);
    };

}
