#include <cppmicrohttpd/config.h>
#include <cppmicrohttpd/request/post_data_request.h>

#include <cppmicrohttpd/request/request.inl>

using namespace ::cppmicrohttpd;

bool post_data_request::handle_post_pata(
    const void *    p_data,
    size_t&         p_size)
{
    /* check data size */
    if (p_size <= 0)
        return false;

    /* lazy initialize the post processor */
    if (!_post_processor)
    {
        _post_processor.reset(MHD_create_post_processor(
            connection,
            _buffer_size,
            &post_data_request::mhd_post_data_iterator_callback,
            this));
        if (!_post_processor)
            throw exception("Unable to create post data processor!");
    }

    /* execute post data processor */
    auto ret = MHD_post_process(_post_processor.get(), static_cast<const char *>(p_data), p_size);
    p_size = 0;
    if (ret != MHD_YES)
        throw exception("Unable to handle post data!");

    // libmicrohttpd requieres to process all post data before enqueue a response
    // so we always return "true" until all data has been processed
    // https://lists.gnu.org/archive/html/libmicrohttpd/2011-09/msg00028.html
    return true;
}

int post_data_request::mhd_post_data_iterator_callback(
    void * cls,
    enum MHD_ValueKind kind,
    const char * key,
    const char * filename,
    const char * content_type,
    const char * transfer_encoding,
    const char * data,
    uint64_t off,
    size_t size)
{
    if (!cls)
        return MHD_NO;

    auto& req = *static_cast<post_data_request*>(cls);
    try
    {
        return req.handle_processed_post_data(kind, key, filename, content_type, transfer_encoding, data, off, size);
    }
    catch(const http_exception& ex)
    {
        cppmicrohttpd_log(warn) << "Error while iterating post data: " << ex;
        req.set_error(ex);
    }
    catch(const exception& ex)
    {
        cppmicrohttpd_log(warn) << "Error while iterating post data: " << ex;
        req.set_error(ex);
    }
    catch(const std::exception& ex)
    {
        cppmicrohttpd_log(warn) << "Error while iterating post data: " << ex.what();
        req.set_error(exception(ex.what()));
    }
    catch(...)
    {
        cppmicrohttpd_log(warn) << "Error while iterating post data: unknown";
        req.set_error(exception("Unknown error"));
    }
    return MHD_NO;
}
