#include <cppmicrohttpd/config.h>
#include <cppmicrohttpd/response/callback_response.h>

using namespace ::cppmicrohttpd;

ssize_t callback_response::mhd_content_reader_callback(
    void * cls,
    uint64_t pos,
    char * buf,
    size_t max)
{
    if (!cls)
        return static_cast<ssize_t>(MHD_CONTENT_READER_END_OF_STREAM);

    auto& res = *static_cast<callback_response*>(cls);
    try
    {
        return res.read_content(pos, buf, max);
    }
    catch(const std::exception& ex)
    {
        cppmicrohttpd_log(warn) << "Error while generating response data: " << ex.what();
    }
    catch(...)
    {
        cppmicrohttpd_log(warn) << "Error while generating response data: unknown";
    }

    /**
     * HACK:   Actually we should return MHD_CONTENT_READER_END_WITH_ERROR here,
     *         but due to a bug in the microhttp library this will end in a deadlock :/
     *         Details: microhttpd recognizes the error and tries to close the connection
     *                  and free the response object. Deadlock occures when locking
     *                  the mutex of the response object, because it is already locked.
     */
    return static_cast<ssize_t>(MHD_CONTENT_READER_END_OF_STREAM);
}
